@@ -36,6 +36,8 @@ void ConfigHandler::setConfig(KScreen::ConfigPtr config)
 {
     m_config = config;
     m_initialConfig = m_config->clone();
+    m_initialControl.reset(new ControlConfig(m_initialConfig));
+
     KScreen::ConfigMonitor::instance()->addConfig(m_config);
     m_control.reset(new ControlConfig(config));
 
@@ -49,6 +51,8 @@ void ConfigHandler::setConfig(KScreen::ConfigPtr config)
         initOutput(output);
     }
     m_lastNormalizedScreenSize = screenSize();
+
+    // TODO: put this into m_initialControl
     m_initialRetention = getRetention();
     Q_EMIT retentionChanged();
 
@@ -67,9 +71,25 @@ void ConfigHandler::setConfig(KScreen::ConfigPtr config)
     Q_EMIT outputModelChanged();
 }
 
+void ConfigHandler::resetScale(const KScreen::OutputPtr &output)
+{
+    // Load scale control (either not set, same or windowing system does not transmit scale).
+    const qreal scale = m_control->getScale(output);
+    if (scale > 0) {
+        output->setScale(scale);
+        for (auto initialOutput : m_initialConfig->outputs()) {
+            if (initialOutput->id() == output->id()) {
+                initialOutput->setScale(scale);
+                break;
+            }
+        }
+    }
+}
+
 void ConfigHandler::initOutput(const KScreen::OutputPtr &output)
 {
     if (output->isConnected()) {
+        resetScale(output);
         m_outputs->add(output);
     }
     connect(output.data(), &KScreen::Output::isConnectedChanged,
@@ -78,7 +98,7 @@ void ConfigHandler::initOutput(const KScreen::OutputPtr &output)
     });
 }
 
-void ConfigHandler::updateInitialConfig()
+void ConfigHandler::updateInitialData()
 {
     m_initialRetention = getRetention();
     connect(new GetConfigOperation(), &GetConfigOperation::finished,
@@ -87,6 +107,10 @@ void ConfigHandler::updateInitialConfig()
             return;
         }
         m_initialConfig = qobject_cast<GetConfigOperation*>(op)->config();
+        for (auto output : m_config->outputs()) {
+            resetScale(output);
+        }
+        m_initialControl.reset(new ControlConfig(m_initialConfig));
         checkNeedsSave();
     });
 }
@@ -128,7 +152,10 @@ void ConfigHandler::checkNeedsSave()
                              || output->pos() != initialOutput->pos()
                              || output->scale() != initialOutput->scale()
                              || output->rotation() != initialOutput->rotation()
-                             || output->replicationSource() != initialOutput->replicationSource();
+                             || output->replicationSource() != initialOutput->replicationSource()
+                             || autoRotate(output) != m_initialControl->getAutoRotate(output)
+                             || autoRotateOnlyInTabletMode(output)
+                                    != m_initialControl->getAutoRotateOnlyInTabletMode(output);
             }
             if (needsSave) {
                 Q_EMIT needsSaveChecked(true);
@@ -262,6 +289,47 @@ void ConfigHandler::setRetention(int retention)
     Q_EMIT changed();
 }
 
+qreal ConfigHandler::scale(const KScreen::OutputPtr &output) const
+{
+    return m_control->getScale(output);
+}
+
+void ConfigHandler::setScale(KScreen::OutputPtr &output, qreal scale)
+{
+    m_control->setScale(output, scale);
+}
+
+KScreen::OutputPtr ConfigHandler::replicationSource(const KScreen::OutputPtr &output) const
+{
+    return m_control->getReplicationSource(output);
+}
+
+void ConfigHandler::setReplicationSource(KScreen::OutputPtr &output,
+                                         const KScreen::OutputPtr &source)
+{
+    m_control->setReplicationSource(output, source);
+}
+
+bool ConfigHandler::autoRotate(const KScreen::OutputPtr &output) const
+{
+    return m_control->getAutoRotate(output);
+}
+
+void ConfigHandler::setAutoRotate(KScreen::OutputPtr &output, bool autoRotate)
+{
+    m_control->setAutoRotate(output, autoRotate);
+}
+
+bool ConfigHandler::autoRotateOnlyInTabletMode(const KScreen::OutputPtr &output) const
+{
+    return m_control->getAutoRotateOnlyInTabletMode(output);
+}
+
+void ConfigHandler::setAutoRotateOnlyInTabletMode(KScreen::OutputPtr &output, bool value)
+{
+    m_control->setAutoRotateOnlyInTabletMode(output, value);
+}
+
 void ConfigHandler::writeControl()
 {
     if (!m_control) {
